package IniConf::Magic;

use strict;
use vars qw($VERSION);
use IniConf;

$VERSION = '0.01';

######################################################################
# Standard tied hash functions
sub TIEHASH 
{
    my($package, $file)=@_;
    my $ini=IniConf->new('-file'=>$file);
    return unless $ini;
    my $self=bless {ini=>$ini, setup=>[], 'values'=>{}}, $package;
    $self->setup('default');
    return $self;
}

sub STORE
{
    my($self, $key, $value)=@_;
    $self->{'values'}{$key}=[$value];
}

sub _substitute
{
    my($self, $v)=@_;
    
    $v=~s(\[(\w+)\])
	     (exists $self->{'values'}{$1} 
                  ? $self->_substitute($self->{'values'}{$1}[0])
                  : '['.$1.']')ge;
    return $v;
}
sub FETCH
{
    my($self, $key)=@_;
    return unless exists $self->{'values'}{$key};
    my $v=$self->{'values'}{$key}[0];
    return $self->_substitute($v);
}

sub FIRSTKEY
{
    my($self)=@_;
    my $a=keys %{$self->{'values'}};
    each %{$self->{'values'}};
}

sub NEXTKEY
{
    my($self, $lastkey)=@_;
    each %{$self->{'values'}};
}

sub EXISTS
{
    my($self, $key)=@_;
    return exists $self->{'values'}{$key};
}

sub DELETE
{
    my($self, $key)=@_;
    delete $self->{'values'}{$key};
}

sub CLEAR
{
    my($self)=@_;
    $self->{setup}=[];
    $self->{'values'}={};
    return;
}

######################################################################
# this is were the "magic" is.
sub setup
{
    my($self, $section)=@_;
    
    my $ini=$self->{ini};
    my $values=$self->{'values'};
    foreach my $key ($ini->Parameters($section))
    {
        $values->{$key}=[$ini->val($section, $key)];
    }
    push @{$self->{setup}}, $section;
}

######################################################################
sub array
{
    my($self, $key)=@_;
    return unless exists $self->{'values'}{$key};
    return map {$self->_substitute($_)} @{$self->{'values'}{$key}};
}
1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

IniConf::Magic - Add some magic to IniConf

=head1 SYNOPSIS

    use IniConf::Magic;
    use vars qw(%config);
    my $config=tie %config, 'IniConf::Magic', 'something.ini';
    %config{refer}=$ENV{'HTTP_REFERER'};
    $config->setup($hostname);
    $config->setup('fr');

    # now we pull values from %config... these are but examples
    open LOG, ">>$config{log}" or die "Unable to open $config{log}: $!\n";
    print qq(<IMG src="$config{rootURL}/images/something.gif">);
    # ... and so on ...

=head1 DESCRIPTION

This module adds 3 types of magic to IniConf : value interpolation, section
inheritance and a tied interface.

Config files often have to change depending on which host a program is
runing on, what language you want your output to use, day of the week, time
of day, phase of the moon and so on.  Often, it is desirable for values in a
configuration file to include other values.  And most configuration modules
for perl only have a function or method interface, thereby requiring you to
load the values into variables and then manipulate the variables.  This
module attempts to do all these things in a DWIM manner.

=head1 INI SYNTAX

The ini file format was invented by Microsoft for system configuration in
windows.  It is an plain text format that has key/value pairs, grouped into
sections.  The format is not rigid in that it is not validated agains a list
of possible section names, key names and so on.  Sadly, it would not last. 
Microsoft abandoned the the ini file with Windows 95 and now uses the
registry, a hierarchial, binary, file format.

=head2 INI SECTION

A section is a group of key/value pairs that is preceeded by the section
name, in []. 

    [default]
    ; ... values for section default
    [fr]
    ; ... values for section fr

The section named 'default' is special to IniConf::Magic.  It is
automatically used when a file is loaded.  Other sections are used with the
L<setup()> method.

=head2 INI KEY/VALUE TUPLES

A tuple is the the key, a equal sign (=), then the value.  The value may be
any string, or a UNIX "here document" (this is an extention to the basic INI
format).

    root=/home/wcamelot
    text=<<EOT
    This is a multi-line value to show off the cool 
    here document feature that IniConf has.
    EOT

A value may also include the content of another value. 

    root=/home/wcamelot
    logdir=[root]/prive/logs
    logthis=[logdir]/this.log
    logthat=[logdir]/that.log

These tuples are equivalent to the following, only much easier to maintain.

    root=/home/wcamelot
    logdir=/home/wcamelot/prive/logs
    logthis=/home/wcamelot/prive/logs/this.log
    logthat=/home/wcamelot/prive/logs/that.log

=head2 INI COMMENTS

A comment is any line starting with a semicolon (;) or hash (#).

    ; This is an ini comment
    # This is also an ini comment.  IniConf allows us to use #
    # for comments because it's less confusing to perl-heads

=head1 TUTORIAL

Combining sections with interpolation gives us a nearly DWIM configuration
file.  

    ; e1.ini
    [default]
    logdir=/tmp
    logthis=[logdir]/this.log
    logthat=[logdir]/that.log
    
    ; values to use on host gerald
    [gerald]
    logdir=/home/wcamelot/prive/logs

    ; values to use on host arthur
    [arthur]
    logdir=/var/logs


    #perl code
    $s=tie %magic_ini, 'IniConf::Magic', 'e1.ini';
    die "Can't use e1.ini" unless $s;
    
    $p=$magic_ini{logthis};     # /tmp/this.log
    $s->setup('gerald');        # use section gerald
    $p=$magic_ini{logthis};     # /home/wcamelot/prive/logs/this.log

    # You can also use this as a quick and dirty template module.
    $magic_ini{logthis}='[root]/something.log';
    $p=$magic_ini{logthis}      # /home/wcamelot/prive/logs/something.log

Another example of using IniConf::Magic as a template:

    ; e2.ini
    [default]
    msg=<<EOM
        This is an extra long message that has [interpolations] 
        and was writen by [person] as an example.
    EOM
    person=someone    

    ; values to use in french
    [fr]
    msg=<<EOM
        Ceci est un exemple plus long qui utilise des [interpolations] 
        et  t crit par [person] pour vous donner une exemple.
    EOM
    persone=quelqu'un
        
    #perl code
    tie(%magic_ini, 'IniConf::Magic', 'e2.ini')
        or die "Can't use e2.ini";
    
    if($0 =~ /_en[^/]*$/) # english version?
    {
        $magic_ini{interpolations}='balls';
    } else
    {
        tied(%magic_ini)->setup('fr');
        $magic_ini{interpolations}='fleurs';
    }

    $magic_ini{person}=$query->param('someone') if $query->param('someone');

    print $magic_ini{msg};

=head1 METHODS

=over 4

=item setup

Tell the IniConf::Magic object ot use another section of the ini file.  When
a new section is used, it's key/value tuples are merged into the current
tuples.  Think of this as the new section inheritling all the tuples from
the previously used sections.  The section named 'default' is automatically
used when the ini file is loaded.

    my $s=tie(%magic_ini, 'IniConf::Magic', 'e2.ini')
            or die "Can't use e2.ini";
    $s->setup('update');
    $s->setup('weather');
    $s->setup('stuff');

    # all the tuples from the sections 'default', 'update', 'weather',
    # and 'stuff' are now available from %magic_in

If more then one section have a tuple with the same key, the one from the
most recently "used" section is used.  So, in the previous example, if
'default' and 'weather' share the key 'billy', $magic_ini{billy} would
return the one from 'weather'.

=back

=head1 BUGS and TODO

This module was writen in one go.  I only had to correct one typo to get
test.pl to run as expected!  Something must be missing.

IniConf will throw a warning about using undef as an array reference if you
do tied(%magic_ini)->setup('my_section'); and the ini file doesn't contain a
section called 'my_section'.  The following patch fixes this.

    --- IniConf.pm~ Tue Jun  1 03:52:33 1999
    +++ IniConf.pm  Tue Jun  1 03:53:00 1999
    @@ -408,7 +408,7 @@
     sub Parameters {
       my $self = shift;
       my $sect = shift;
    -  @{$self->{parms}{$sect}};
    +  @{$self->{parms}{$sect}||[]};
     }

FETCH only returns a scalar.  Multiple tuples with the same name within a
section or multi-line values can also be seen as an array.

It would be nice if list values could be returned as an array.  Like the
following :
    
    ; e3.ini
    ; ...
    ports=10,20,30,40,50

    # perl code
    @ports=$magic_ini{ports};

It would be nice if things line \n or 0xe9 or \009 would be interpreted.

Maybe we could also load other ini files, sort of like an "include"
directive.  Would a section in the new file completely get rid of a currently
known section of the same name?  Would it's tuples be added to the current 
section ?  (This would probably require modification to IniConf).  If we get
rid of the currently known section, should we also drop tuples that are
being used as well?

We should respond to SIGHUP by rereading the ini file and merging the used
sections again.

=head1 AUTHOR

Philip Gwyn <ini@pied.nu>

=head1 SEE ALSO

L<IniConf>

=cut
